@@ -943,52 +943,6 @@ class Proxy< diagview<eT> >
 
 
 
-template<typename eT>
-class Proxy< spdiagview<eT> >
-  {
-  public:
-  
-  typedef eT                                       elem_type;
-  typedef typename get_pod_type<elem_type>::result pod_type;
-  typedef spdiagview<eT>                           stored_type;
-  typedef const spdiagview<eT>&                    ea_type;
-  typedef const spdiagview<eT>&                    aligned_ea_type;
-  
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
-  static const bool fake_mat    = false;
-  
-  static const bool is_row = false;
-  static const bool is_col = true;
-  
-  arma_aligned const spdiagview<eT>& Q;
-  
-  inline explicit Proxy(const spdiagview<eT>& A)
-    : Q(A)
-    {
-    arma_extra_debug_sigprint();
-    }
-  
-  arma_inline uword get_n_rows() const { return Q.n_rows; }
-  arma_inline uword get_n_cols() const { return 1;        }
-  arma_inline uword get_n_elem() const { return Q.n_elem; }
-  
-  arma_inline elem_type operator[] (const uword i)                const { return Q[i];         }
-  arma_inline elem_type at         (const uword row, const uword) const { return Q.at(row, 0); }
-  arma_inline elem_type at_alt     (const uword i)                const { return Q[i];         }
-  
-  arma_inline         ea_type         get_ea() const { return Q; }
-  arma_inline aligned_ea_type get_aligned_ea() const { return Q; }
-  
-  template<typename eT2>
-  arma_inline bool is_alias(const Mat<eT2>&) const { return false; }
-  
-  arma_inline bool is_aligned() const { return false; }
-  };
-
-
-
 template<typename T1>
 class Proxy_diagvec_mat
   {
