@@ -1,11 +1,17 @@
-// Copyright (C) 2013-2014 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup arma_rng_cxx98
@@ -140,18 +146,18 @@ arma_rng_cxx98::randi_fill(eT* mem, const uword N, const int a, const int b)
     {
     for(uword i=0; i<N; ++i)
       {
-      mem[i] = std::rand();
+      mem[i] = eT(std::rand());
       }
     }
   else
     {
-    const uword length = b - a + 1;
+    const uword length = uword(b - a + 1);
     
     const double scale = double(length) / double(randi_max_val());
     
     for(uword i=0; i<N; ++i)
       {
-      mem[i] = (std::min)( b, (int( double(randi_val()) * scale ) + a) );
+      mem[i] = eT((std::min)( b, (int( double(randi_val()) * scale ) + a) ));
       }
     }
   }
