@@ -198,4 +198,14 @@ class spop_sign
 
 
 
+class spop_diagvec
+  {
+  public:
+  
+  template<typename T1>
+  inline static void apply(SpMat<typename T1::elem_type>& out, const SpOp<T1, spop_diagvec>& in);
+  };
+
+
+
 //! @}
